from typing import List

from langchain_community.chat_message_histories import SQLChatMessageHistory
from langchain_core.chat_history import BaseChatMessageHistory
from langchain_core.messages import BaseMessage, HumanMessage, AIMessage

from bot_station.domain.bot.chat_message_storage import ChatMessageStorage
from bot_station.domain.bot.model.lm_chat_message import (
    LmChatMessage,
    LMUserMessage,
    LMBotMessage,
)


class ChatMessageStorageImpl(ChatMessageStorage):
    root_message_history_path = ""

    def __init__(self, root_message_history_path: str):
        self.root_message_history_path = root_message_history_path

    async def get_history(
        self, chat_id: int | str, limit: int = 100
    ) -> List[LmChatMessage]:
        base_messages: List[BaseMessage] = self.__get_message_history(chat_id).messages
        messages: List[LmChatMessage] = []
        counter = 0
        for m in base_messages:
            if counter > limit:
                break
            if isinstance(m, HumanMessage):
                messages.append(LMUserMessage(chat_id=chat_id, text=m.content))
            if isinstance(m, AIMessage):
                messages.append(LMBotMessage(chat_id=chat_id, text=m.content))
            counter = counter + 1
        return messages

    async def add_bot_message(self, chat_id: int | str, message: LmChatMessage):
        chat_message_history = self.__get_message_history(chat_id)
        chat_message_history.add_ai_message(message.text)

    async def add_user_message(self, chat_id: int | str, message: LmChatMessage):
        chat_message_history = self.__get_message_history(chat_id)
        chat_message_history.add_user_message(message.text)

    def get_message_history(self, chat_id: int | str) -> BaseChatMessageHistory:
        return self.__get_message_history(chat_id)

    async def clear(self, chat_id: int | str):
        pass

    def __get_message_history(self, chat_id: int | str) -> SQLChatMessageHistory:
        return SQLChatMessageHistory(
            session_id=str(chat_id),
            connection_string=f"sqlite:///{self.root_message_history_path}/chat-{str(chat_id)}.db",
        )



from abc import abstractmethod, ABC
from typing import List

from bot_station.domain.bot.bot import Bot
from bot_station.domain.bot.model.bot_meta_info import BotMetaInfo


class BotStation(ABC):

    @abstractmethod
    async def create(self, config: BotMetaInfo):
        pass

    @abstractmethod
    async def get_bot(self, bot_id: str) -> Bot | None:
        pass

    @abstractmethod
    async def get_bots_list(self) -> List[BotMetaInfo]:
        pass

    @abstractmethod
    async def delete(self, bot_id: str) -> bool:
        pass

    @abstractmethod
    async def update(self, info: BotMetaInfo):
        pass



from abc import ABC
from dataclasses import dataclass


class LmChatMessage(ABC):
    """
    @:param chat_id external connector chat id
    """

    text: str
    chat_id: int | str = None


@dataclass
class LMUserMessage(LmChatMessage):
    text: str
    chat_id: int | str


@dataclass
class LMBotMessage(LmChatMessage):
    text: str
    chat_id: int | str



from dataclasses import dataclass

from pydantic import BaseModel, Field


@dataclass
class BotCallDto(BaseModel):
    bot_id: str = Field(min_length=1, max_length=100)
    chat_id: int | str = Field()
    data: str = Field(min_length=0, max_length=10_000)



import logging
from typing import Any, Callable

import uvicorn
from fastapi import APIRouter, FastAPI, Request, Response
from fastapi.routing import APIRoute
from starlette import status

from bot_station.api.rest.model.bot_call_dto import BotCallDto
from bot_station.api.rest.model.bot_call_result import (
    BotCallResult,
    BotNotFoundCallResult,
    BotAnswerCallResult,
)
from bot_station.api.rest.model.bot_creation_dto import BotInfoDto
from bot_station.api.rest.model.bot_dto import BotDto
from bot_station.api.rest.model.bot_train_dto import BotTrainDto
from bot_station.api.rest.model.web_app_config import WebAppConfig
from bot_station.data.bot.rag_bot_impl import RAGBot
from bot_station.domain.bot.bot import Bot
from bot_station.domain.bot.model.bot_meta_info import BotMetaInfo
from bot_station.domain.bot.model.lm_chat_message import LMUserMessage
from bot_station.domain.bot_station.bot_station import BotStation
from bot_station.domain.docs.docs_library import DocsLibrary
from bot_station.domain.docs.model.document import Document


class LoggingRoute(APIRoute):
    def get_route_handler(self) -> Callable:
        original_route_handler = super().get_route_handler()

        async def custom_route_handler(request: Request) -> Response:
            logging.debug(
                f"--> {request.method} {request.url}\nBody: {await request.body()};\nHeaders: {request.headers}"
            )
            response: Response = await original_route_handler(request)
            logging.debug(f"<-- {response.status_code} {response.body}")
            return response

        return custom_route_handler


router = APIRouter(route_class=LoggingRoute)


@router.post("/create", status_code=200)
async def create(*, dto: BotInfoDto) -> Any:
    logging.debug(f"/create dto : {dto}")
    bot_id: str | None = await BotStationWebApp.create(dto)
    if bot_id is not None:
        return Response(status_code=status.HTTP_200_OK)
    else:
        return Response(status_code=status.HTTP_500_INTERNAL_SERVER_ERROR)


@router.post("/update", status_code=200)
async def update(*, dto: BotInfoDto) -> Any:
    logging.debug(f"/create dto : {dto}")
    bot_id: str | None = await BotStationWebApp.update(dto)
    if bot_id is not None:
        return Response(status_code=status.HTTP_200_OK)
    else:
        return Response(status_code=status.HTTP_500_INTERNAL_SERVER_ERROR)


@router.post("/train", status_code=200, description="Returns ID of new document")
async def train(*, dto: BotTrainDto) -> Any:
    logging.debug(f"/train dto : {dto}")
    try:
        doc: Document = await BotStationWebApp.train(dto)
        return doc.id
    except ValueError:
        logging.debug(f"incorrect dto : {dto}")
        return Response(status_code=status.HTTP_404_NOT_FOUND)


@router.post("/call", status_code=200, response_model=BotAnswerCallResult)
async def call(*, dto: BotCallDto) -> Any:
    logging.debug(f"/call dto : {dto}")
    result: BotCallResult = await BotStationWebApp.call(dto)
    if isinstance(result, BotNotFoundCallResult):
        return Response(status_code=status.HTTP_404_NOT_FOUND)
    if isinstance(result, BotAnswerCallResult):
        logging.debug(f"/call return response : {result}")
        return result
    return Response(status_code=status.HTTP_500_INTERNAL_SERVER_ERROR)


@router.get("/admin/bots", status_code=200, response_model=list[BotDto])
async def get_all_bots() -> Any:
    logging.debug("/admin/bots")
    result = await BotStationWebApp.get_all_bots()
    logging.debug(f"/admin/bots count {len(result)}")
    return result


@router.delete("/admin/{bot_id}", status_code=200)
async def delete(*, bot_id: str) -> Any:
    logging.debug(f"/delete {bot_id}")
    is_deleted = await BotStationWebApp.delete(bot_id=bot_id)
    if is_deleted:
        return Response(status_code=status.HTTP_200_OK)
    else:
        return Response(status_code=status.HTTP_404_NOT_FOUND)


def run_server():
    uvicorn.run(BotStationWebApp.api, host="0.0.0.0", port=8000)


class BotStationWebApp(object):
    api: FastAPI
    bot_station: BotStation = None

    @staticmethod
    def prepare(bot_station: BotStation, config: WebAppConfig):
        BotStationWebApp.bot_station = bot_station
        BotStationWebApp.api = FastAPI(
            title=config.title, debug=config.debug, version=config.version
        )
        BotStationWebApp.api.include_router(router)

    @staticmethod
    def launch():
        run_server()

    @staticmethod
    async def create(dto: BotInfoDto) -> str | None:
        meta_info = BotMetaInfo(
            id=dto.id,
            name=dto.name,
            description=dto.description,
            prompt_intro=dto.prompt_intro,
            add_external_context_to_prompt=dto.add_external_context_to_prompt,
            add_messages_history_to_prompt=dto.add_messages_history_to_prompt,
            temperature=dto.temperature,
        )
        try:
            await BotStationWebApp.bot_station.create(meta_info)
            return meta_info.id
        except Exception as e:
            logging.warning(e)
            return None

    @staticmethod
    async def update(dto: BotInfoDto) -> str | None:
        meta_info = BotMetaInfo(
            id=dto.id,
            name=dto.name,
            description=dto.description,
            prompt_intro=dto.prompt_intro,
            add_external_context_to_prompt=dto.add_external_context_to_prompt,
            add_messages_history_to_prompt=dto.add_messages_history_to_prompt,
            temperature=dto.temperature,
        )
        try:
            await BotStationWebApp.bot_station.update(meta_info)
            return meta_info.id
        except Exception as e:
            logging.warning(e)
            return None

    @staticmethod
    async def train(dto: BotTrainDto) -> Document:
        bot: RAGBot | None = await BotStationWebApp.bot_station.get_bot(bot_id=dto.bot_id)
        if bot is None or not isinstance(bot, RAGBot) or not isinstance(bot.docs_source, DocsLibrary):
            raise ValueError(f"Bot with id {dto.bot_id} not exist")
        else:
            await bot.docs_source.load()
            doc = await bot.docs_source.create_document(
                data=dto.data,
                source_link=dto.source_link,
                metadata=dto.metadata,
            )
            await bot.docs_source.close()
            return doc

    @staticmethod
    async def call(dto: BotCallDto) -> BotCallResult:
        bot: Bot | None = await BotStationWebApp.bot_station.get_bot(bot_id=dto.bot_id)
        if bot is None:
            return BotNotFoundCallResult()
        else:
            text = dto.data
            call_result = await bot.call(
                question=LMUserMessage(text=text, chat_id=dto.chat_id)
            )
            answer_text = call_result.answer.text
            return BotAnswerCallResult(
                text=answer_text, relevant_docs=call_result.relevant_docs
            )

    @staticmethod
    async def get_all_bots() -> list[BotDto]:
        bots_info = await BotStationWebApp.bot_station.get_bots_list()
        return [
            BotDto(
                id=b.id,
                name=b.name,
                description=b.description,
                prompt_intro=b.prompt_intro,
                add_external_context_to_prompt=b.add_external_context_to_prompt,
                add_messages_history_to_prompt=b.add_messages_history_to_prompt,
                temperature=b.temperature,
            )
            for b in bots_info
        ]

    @staticmethod
    async def delete(bot_id: str) -> bool:
        return await BotStationWebApp.bot_station.delete(bot_id=bot_id)



from pydantic import BaseModel, Field

from bot_station.domain.docs.model.document import Document


class BotCallResult(BaseModel):
    pass


class BotNotFoundCallResult(BotCallResult):
    pass


class BotAnswerCallResult(BotCallResult, BaseModel):
    text: str = Field()
    relevant_docs: list[Document] = Field(default=None)



import uuid

from sqlalchemy import types
from sqlalchemy.dialects.mysql.base import MSBinary
from sqlalchemy.schema import Column


class UUID(types.TypeDecorator):
    impl = MSBinary

    def __init__(self):
        self.impl.length = 16
        types.TypeDecorator.__init__(self, length=self.impl.length)

    def process_bind_param(self, value, dialect=None):
        if value and isinstance(value, uuid.UUID):
            return value.bytes
        elif value and isinstance(value, str):
            try:
                return uuid.UUID(value).bytes
            except ValueError:
                return None
        else:
            return None

    def process_result_value(self, value, dialect=None):
        if value:
            return uuid.UUID(bytes=value)
        else:
            return None

    def is_mutable(self):
        return False


def uuid_id_column(column_name="id"):
    return Column(column_name, UUID(), primary_key=True, default=uuid.uuid4)



from abc import abstractmethod
from typing import Iterable

from bot_station.domain.docs.docs_source import DocsSource
from bot_station.domain.docs.model.document import Document, DocId


class DocsLibrary(DocsSource):

    @abstractmethod
    async def create_document(
            self,
            data: str,
            source_link: str | None = None,
            metadata: dict = {},
    ) -> Document:
        pass

    @abstractmethod
    async def delete_document(self, doc_id: DocId):
        pass

    @abstractmethod
    async def get_all_docs(self) -> Iterable[Document]:
        pass

    @abstractmethod
    async def clear(self):
        pass



from bot_station.domain.llm.llm import LLM


class MockLLM(LLM):
    next_answer = ""

    async def call(self, query: str) -> str:
        return self.next_answer

    def set_next_answer(self, answer: str):
        self.next_answer = answer



from abc import ABC, abstractmethod
from typing import List

from bot_station.domain.bot.model.bot_meta_info import BotMetaInfo


class BotRegistry(ABC):

    @abstractmethod
    async def create(self, config: BotMetaInfo):
        pass

    @abstractmethod
    async def update(self, meta: BotMetaInfo):
        pass

    @abstractmethod
    async def get(self, bot_id: str) -> BotMetaInfo | None:
        pass

    @abstractmethod
    async def get_all(self) -> List[BotMetaInfo]:
        pass

    @abstractmethod
    async def delete(self, bot_id: str):
        pass



from abc import ABC, abstractmethod

from bot_station.domain.docs.model.documents_search_result import DocumentsSearchResult


class DocsSource(ABC):

    @abstractmethod
    async def load(self):
        pass

    @abstractmethod
    async def get_relevant_docs(
            self,
            query: str,
            min_score: float = 0.8,
            limit: int = 10
    ) -> DocumentsSearchResult:
        pass

    @abstractmethod
    async def close(self):
        pass



from qdrant_client.fastembed_common import QueryResponse

from bot_station.domain.docs.model.document import Document

SOURCE_LINK_KEY = "source_link"
CUSTOM_ID = "custom_id"
ID_KEY = "id"


class DocumentMapper:
    @staticmethod
    def from_qdrant_response(doc: QueryResponse) -> Document:
        return Document(
            data=doc.document,
            metadata=doc.metadata,
            source_link=doc.metadata.get(SOURCE_LINK_KEY, None),
            id=doc.id,
        )



from abc import ABC, abstractmethod
from typing import List

from langchain_core.chat_history import BaseChatMessageHistory

from bot_station.domain.bot.model.lm_chat_message import LmChatMessage


class ChatMessageStorage(ABC):

    @abstractmethod
    async def add_user_message(self, chat_id: int | str, message: LmChatMessage):
        pass

    @abstractmethod
    async def add_bot_message(self, chat_id: int | str, message: LmChatMessage):
        pass

    @abstractmethod
    async def get_history(
        self, chat_id: int | str, limit: int = 100
    ) -> List[LmChatMessage]:
        pass

    @abstractmethod
    def get_message_history(self, chat_id: int | str) -> BaseChatMessageHistory:
        pass

    @abstractmethod
    async def clear(self, chat_id: int | str):
        pass



import unittest

from bot_station.data.bot.model.qdrant_config import QdrantConfig
from bot_station.data.bot.model.yandex_cloud_config import YandexCloudConfig
from bot_station.data.bot.rag_bot_impl import RAGBot
from bot_station.data.bot_station.bot_station_impl import BotStationImpl
from bot_station.data.docs.qdrant_docs_library import QdrantDocsLibrary
from bot_station.domain.bot.model.bot_meta_info import BotMetaInfo
from bot_station.domain.bot.model.lm_chat_message import LMUserMessage
from bot_station.domain.bot_station.bot_station import BotStation
from bot_station.domain.docs.model.documents_search_result import DocumentsSearchResult
from tests.mock_llm import MockLLM


class BotStationTest(unittest.IsolatedAsyncioTestCase):
    yandex_cloud_config = YandexCloudConfig(
        api_key="",
        folder_id="",
        model_name="",
        model_version="rc"
    )
    qdrant_config = QdrantConfig(
        qdrant_url=None,
        qdrant_db_path=".data/qdrant",
    )
    docs_library = QdrantDocsLibrary(
        collection_name="",
        qdrant_config=qdrant_config,
    )
    llm = MockLLM()

    bot_station: BotStation = BotStationImpl(yandex_cloud_config=yandex_cloud_config,
                                             qdrant_config=qdrant_config)
    bot_station.llm = llm

    bot_id = "test"
    meta_info = BotMetaInfo(
        name="Nutritio",
        description="Бот, отвечающий на вопросы о правильном питании",
        prompt_intro="Ты - нутрициолог, помогающий выстроить правильное питание.",
        add_external_context_to_prompt=True,
        add_messages_history_to_prompt=False,
        temperature=0.1,
        id=bot_id,
        min_extract_relevant_docs_score=0.8
    )

    async def asyncSetUp(self):
        await self.__delete_bot(self.bot_id)

    async def asyncTearDown(self):
        await self.__delete_bot(self.bot_id)

    async def test_create_bot(self):
        await self.bot_station.create(self.meta_info)
        bot = await self.bot_station.get_bot(self.bot_id)

        self.assertIsNotNone(bot)
        self.assertEqual(self.meta_info.min_extract_relevant_docs_score, bot.meta.min_extract_relevant_docs_score)
        await self.__delete_bot(self.bot_id)

    async def test_train_bot(self):
        await self.bot_station.create(self.meta_info)
        bot: RAGBot | None = await self.bot_station.get_bot(self.bot_id)
        await bot.docs_source.load()

        doc = await bot.docs_source.create_document(
            data="FastAPI is a modern, fast, web framework for building APIs with Python.")
        answer = await bot.call(question=LMUserMessage("What is FastAPI?", chat_id=0))
        relevant_doc = answer.relevant_docs[0]

        self.assertIsNotNone(answer)
        self.assertEqual(doc.id, relevant_doc.id)
        await self.__delete_bot(self.bot_id)

    async def test_fetch_all_docs(self):
        await self.bot_station.create(self.meta_info)
        bot: RAGBot | None = await self.bot_station.get_bot(self.bot_id)
        await bot.docs_source.load()

        await bot.docs_source.create_document(
            data="FastAPI is a modern, fast, web framework for building APIs with Python.",
            source_link="http://link.co")
        await bot.docs_source.create_document(
            data="FastAPI is a web framework for hackathon developers.")

        docs_iterable = await bot.docs_source.get_all_docs()
        all_docs = list(docs_iterable)

        self.assertEqual(2, len(all_docs))
        await self.__delete_bot(self.bot_id)

    async def test_fetch_relevant_docs(self):
        await self.bot_station.create(self.meta_info)
        bot: RAGBot | None = await self.bot_station.get_bot(self.bot_id)
        await bot.docs_source.load()

        await bot.docs_source.create_document(
            data="FastAPI is a modern, fast, web framework for building APIs with Python language.",
            source_link="http://link.co")
        await bot.docs_source.create_document(
            data="FastAPI is a web framework for hackathon developers.")

        search_result: DocumentsSearchResult = await bot.docs_source.get_relevant_docs(
            "What programming language can used for FastAPI?"
        )
        self.assertEqual(2, len(search_result.docs))
        await self.__delete_bot(self.bot_id)

    async def __delete_bot(self, bot_id: str):
        await self.bot_station.delete(bot_id=bot_id)


if __name__ == '__main__':
    unittest.main()



from abc import abstractmethod, ABC

from bot_station.domain.bot.model.bot_meta_info import BotMetaInfo
from bot_station.domain.bot.model.lm_call_result import CallResult
from bot_station.domain.bot.model.lm_chat_message import LMUserMessage


class Bot(ABC):
    meta: BotMetaInfo

    @abstractmethod
    async def call(self, question: LMUserMessage) -> CallResult:
        pass



from dataclasses import dataclass


@dataclass
class RAGPromptSamples:
    message_history_title: str = "Переписка с пользователем: "
    relevant_docs_title: str = "Для ответа используй следующую информацию: "
    user_message_title: str = "Сообщение пользователя: "



from langchain_community.chat_models import ChatYandexGPT
from langchain_core.tracers import langchain

from bot_station.data.bot.model.yandex_cloud_config import YandexCloudConfig
from bot_station.domain.llm.llm import LLM


class YandexGPTLLM(LLM):

    def __init__(self, yandex_cloud_config: YandexCloudConfig):
        langchain.debug = False
        self.yandex_cloud_config = yandex_cloud_config

    async def call(self, query: str, temperature: float = 0.6) -> str:
        gpt = ChatYandexGPT(
            api_key=self.yandex_cloud_config.api_key,
            folder_id=self.yandex_cloud_config.folder_id,
            temperature=temperature,
            model_name=self.yandex_cloud_config.model_name,
            model_version=self.yandex_cloud_config.model_version,
            verbose=True,
        )
        return gpt.invoke(query).content



import logging
from typing import List

from bot_station.data.bot.chat_message_storage_impl import ChatMessageStorageImpl
from bot_station.data.bot.model.qdrant_config import QdrantConfig
from bot_station.data.bot.model.yandex_cloud_config import YandexCloudConfig
from bot_station.data.bot.rag_bot_impl import RAGBot
from bot_station.data.bot_station.bot_registry_impl import BotRegistryImpl
from bot_station.data.docs.qdrant_docs_library import QdrantDocsLibrary
from bot_station.data.llm.yandex_gpt_llm import YandexGPTLLM
from bot_station.domain.base.const import message_history_path
from bot_station.domain.bot.bot import Bot
from bot_station.domain.bot.model.bot_meta_info import BotMetaInfo
from bot_station.domain.bot_station.bot_station import BotStation
from bot_station.domain.docs.docs_library import DocsLibrary


class BotStationImpl(BotStation):
    __in_memory_bots: dict[str, RAGBot] = {}

    def __init__(self,
                 yandex_cloud_config: YandexCloudConfig,
                 qdrant_config: QdrantConfig,
                 ):
        self.bot_registry = BotRegistryImpl()
        self.yandex_cloud_config = yandex_cloud_config
        self.llm = YandexGPTLLM(yandex_cloud_config=self.yandex_cloud_config)
        self.qdrant_config = qdrant_config

    async def create(self, bot_info: BotMetaInfo):
        logging.debug(f"create {bot_info}")
        bot_with_same_id = await self.bot_registry.get(bot_id=bot_info.id)
        if bot_with_same_id is None:
            meta_info = await self.bot_registry.create(bot_info)
            return meta_info
        else:
            raise Exception(f"Bot with id '{bot_info.id}' already exists!")

    async def get_bot(self, bot_id: str) -> Bot | None:
        logging.debug(f"Get {bot_id})")
        if bot_id is None:
            raise Exception("Bot id is None!")
        bot = self.__in_memory_bots.get(bot_id, None)
        if bot is not None:
            return bot

        meta = await self.bot_registry.get(bot_id)
        if meta is None:
            logging.warning(f"No {bot_id} in registry!")
            return None
        else:
            docs_library = QdrantDocsLibrary(
                collection_name=meta.id,
                qdrant_config=self.qdrant_config
            )
            bot = RAGBot(
                message_storage=ChatMessageStorageImpl(root_message_history_path=message_history_path),
                llm=self.llm,
                docs_source=docs_library,
                meta=meta
            )
            await docs_library.load()
            self.__in_memory_bots[bot_id] = bot
            return bot

    async def get_bots_list(self) -> List[BotMetaInfo]:
        logging.debug("get_bots_list")
        return await self.bot_registry.get_all()

    async def delete(self, bot_id: str) -> bool:
        logging.debug(f"Delete {bot_id}")
        bot: RAGBot | None = await self.get_bot(bot_id=bot_id)
        logging.debug(f"Delete bot {bot}")
        if bot is not None:
            if isinstance(bot, RAGBot) and isinstance(bot.docs_source, DocsLibrary):
                logging.debug(f"Clear library {bot_id}")
                await bot.docs_source.load()
                await bot.docs_source.clear()
                await bot.docs_source.close()
            await self.bot_registry.delete(bot_id=bot_id)
            self.__in_memory_bots.pop(bot_id, None)
            return True
        else:
            logging.warning(f"No {bot_id} in registry")
            return False

    async def update(self, info: BotMetaInfo):
        logging.debug(f"Update {info}")
        await self.bot_registry.update(info)
        # reload bot
        bot = self.__in_memory_bots.pop(info.id, None)
        if bot is not None:
            logging.debug(f"Close {bot.meta}")
            await bot.docs_source.close()



import logging
import unittest

from fastapi.testclient import TestClient

from bot_station.api.rest.api import BotStationWebApp
from bot_station.api.rest.model.web_app_config import WebAppConfig
from bot_station.data.bot.model.qdrant_config import QdrantConfig
from bot_station.data.bot.model.yandex_cloud_config import YandexCloudConfig
from bot_station.data.bot_station.bot_station_impl import BotStationImpl
from bot_station.data.docs.qdrant_docs_library import QdrantDocsLibrary
from bot_station.domain.bot_station.bot_station import BotStation
from mock_llm import MockLLM

logging.basicConfig(
    format="%(asctime)s.%(msecs)06f %(levelname)-8s [%(filename)s:%(lineno)d] %(message)s",
    datefmt="%d-%m-%Y:%H:%M:%S",
    level=logging.DEBUG,
)


def launch_test_web_app(llm: MockLLM):
    yandex_cloud_config = YandexCloudConfig(
        api_key="",
        folder_id="",
        model_name="",
        model_version=""
    )
    qdrant_config = QdrantConfig(
        qdrant_url=None,
        qdrant_db_path=".data/qdrant",
    )

    bot_station: BotStation = BotStationImpl(yandex_cloud_config=yandex_cloud_config,
                                             qdrant_config=qdrant_config)
    bot_station.llm = llm
    BotStationWebApp.prepare(bot_station=bot_station, config=WebAppConfig())


class BotStationApiTest(unittest.TestCase):
    bot_station_test_client: TestClient
    docs_library = QdrantDocsLibrary(
        collection_name="",
        qdrant_config=QdrantConfig(
            qdrant_url=None,
            qdrant_db_path="./.data/qdrant")
    )
    llm = MockLLM()

    def setUp(self):
        launch_test_web_app(llm=self.llm)
        self.bot_station_test_client = TestClient(BotStationWebApp.api)

    def create(self, bot_id: str, check_response: bool = False):
        response = self.bot_station_test_client.post(
            url="/create",
            json={
                "id": bot_id,
                "name": "name",
                "description": "description",
                "prompt_intro": "You are programmer's assistant",
                "add_external_context_to_prompt": "true",
                "add_messages_history_to_prompt": "true",
                "temperature": 0.6
            }
        )
        logging.debug(f"Response = {response.content}")
        if check_response:
            self.assertEqual(200, response.status_code)

    def delete(self, bot_id: str, check_response: bool = False):
        response = self.bot_station_test_client.delete(url=f"/admin/{bot_id}")
        logging.debug(f"response = {response.content}")
        if check_response:
            self.assertEqual(200, response.status_code)

    def test_train(self):
        bot_id = "test_train"
        self.create(bot_id=bot_id, check_response=False)
        response = self.bot_station_test_client.post(
            url="/train",
            json={
                "bot_id": bot_id,
                "data": "FastAPI is a modern, fast, web framework for building APIs with Python.",
                "source_link": "https://aaa.ru"
            }
        )
        self.assertEqual(200, response.status_code)
        self.assertTrue(len(response.content) > 0)
        self.delete(bot_id=bot_id, check_response=True)

    def test_train_with_metadata(self):
        bot_id = "test_train_with_metadata"
        self.create(bot_id=bot_id, check_response=False)
        response = self.bot_station_test_client.post(
            url="/train",
            json={
                "bot_id": bot_id,
                "data": "FastAPI is a modern, fast, web framework for building APIs with Python.",
                "source_link": "https://aaa.ru",
                "metadata": {"a": "b"}
            }
        )
        self.assertEqual(200, response.status_code)
        self.delete(bot_id=bot_id, check_response=True)

    def test_train_incorrect_id(self):
        response = self.bot_station_test_client.post(
            url="/train",
            json={
                "bot_id": "-",
                "id": "1",
                "data": "FastAPI is a modern, fast, web framework for building APIs with Python.",
                "source_link": "https://aaa.ru"
            }
        )
        self.assertEqual(404, response.status_code)

    def test_call(self):
        answer = "FastAPI is a modern, web framework"

        bot_id = "test_call"
        self.create(bot_id=bot_id, check_response=False)
        self.llm.set_next_answer(answer)
        response = self.bot_station_test_client.post(
            url="/call",
            json={
                "bot_id": bot_id,
                "chat_id": 1,
                "data": "What is FastAPI?"
            }
        )
        logging.debug(f"response = {response.content}")
        self.assertEqual(200, response.status_code)
        self.assertEqual(answer, response.json()["text"])
        self.delete(bot_id=bot_id, check_response=True)

    def test_update(self):
        bot_id = "test_update"
        self.create(bot_id=bot_id, check_response=False)

        response = self.bot_station_test_client.post(
            url="/update",
            json={
                "id": bot_id,
                "name": "name",
                "description": "description",
                "prompt_intro": "You are programmer's assistant",
                "add_external_context_to_prompt": "true",
                "add_messages_history_to_prompt": "true",
                "temperature": 0.6
            }
        )
        logging.debug(f"response = {response.content}")
        self.assertEqual(200, response.status_code)


if __name__ == '__main__':
    unittest.main()
